@@ -13,10 +13,7 @@ use curve25519_dalek::{
 };
 use data_rows::DataRow;
 use okvs::schemes::{paxos::Paxos, Okvs};
-use pyo3::{
-    prelude::*,
-    types::{PyBytes, PyTuple},
-};
+use pyo3::{prelude::*, types::PyTuple};
 use rand::thread_rng;
 use rayon::iter::IndexedParallelIterator;
 use rayon::iter::ParallelIterator;
@@ -35,24 +32,18 @@ fn key_gen(py: Python) -> &PyTuple {
     let pk_bytes = pk.compress().as_bytes().to_vec();
     let sk_bytes = sk.as_bytes().to_vec();
 
-    let pk_object: PyObject = PyBytes::new(py, &pk_bytes).into();
-    let sk_object: PyObject = PyBytes::new(py, &sk_bytes).into();
-
-    PyTuple::new(py, [pk_object, sk_object])
+    PyTuple::new(py, [pk_bytes, sk_bytes])
 }
 
-fn read_public_key(py: Python, public_key: PyObject) -> EdwardsPoint {
-    let public_key_bytes: &PyBytes = public_key
-        .cast_as(py)
-        .expect("The public key object should be of type `bytes`.");
-    CompressedEdwardsY::from_slice(public_key_bytes.as_bytes())
+fn read_public_key(_py: Python, public_key: Vec<u8>) -> EdwardsPoint {
+    CompressedEdwardsY::from_slice(&public_key)
         .decompress()
         .expect("The public key was not a valid curve point.")
 }
 
 /// Builds an OKVS for one party. Takes the public key of this party and the data from the database, which is a list of rows. Each row is a list of the relevant columns as a String.
 #[pyfunction]
-fn build_okvs(py: Python, public_key: PyObject, strings_in_each_row: Vec<Vec<String>>) -> PyObject {
+fn build_okvs(py: Python, public_key: Vec<u8>, strings_in_each_row: Vec<Vec<String>>) -> Vec<u8> {
     let public_key_table = EdwardsBasepointTable::create(&read_public_key(py, public_key));
 
     let data_rows: Vec<(DataRow, Ciphertext)> = strings_in_each_row
@@ -63,7 +54,7 @@ fn build_okvs(py: Python, public_key: PyObject, strings_in_each_row: Vec<Vec<Str
     // TODO: Parallelize Paxos
     let okvs: Paxos<Ciphertext> = Paxos::encode(&data_rows, 40);
 
-    PyBytes::new(py, &okvs.to_bytes()).into()
+    okvs.to_bytes()
 }
 
 #[pyfunction]
@@ -71,23 +62,16 @@ fn initiate_queries(
     py: Python,
     sender_data: Vec<Vec<String>>,
     receiver_data: Vec<Vec<String>>,
-    sender_okvs: PyObject,
-    receiver_okvs: PyObject,
-    swift_pk: PyObject,
-) -> PyObject {
-    let sender_okvs_bytes: &PyBytes = sender_okvs
-        .cast_as(py)
-        .expect("The sender_okvs object should be of type `bytes`.");
-    let sender_okvs_instance: Paxos<Ciphertext> = Paxos::from_bytes(sender_okvs_bytes.as_bytes());
+    sender_okvs: Vec<u8>,
+    receiver_okvs: Vec<u8>,
+    swift_pk: Vec<u8>,
+) -> Vec<u8> {
+    let sender_okvs_instance: Paxos<Ciphertext> = Paxos::from_bytes(&sender_okvs);
     let sender_ciphertexts = sender_data
         .into_par_iter()
         .map(|row| sender_okvs_instance.decode(&DataRow(row)));
 
-    let receiver_okvs_bytes: &PyBytes = receiver_okvs
-        .cast_as(py)
-        .expect("The receiver_okvs object should be of type `bytes`.");
-    let receiver_okvs_instance: Paxos<Ciphertext> =
-        Paxos::from_bytes(receiver_okvs_bytes.as_bytes());
+    let receiver_okvs_instance: Paxos<Ciphertext> = Paxos::from_bytes(&receiver_okvs);
     let receiver_ciphertexts = receiver_data
         .into_par_iter()
         .map(|row| receiver_okvs_instance.decode(&DataRow(row)));
@@ -98,36 +82,24 @@ fn initiate_queries(
         .flat_map(|(cs, cr)| Ciphertext::add_and_randomize(cs, cr, &public_key).to_bytes())
         .collect();
 
-    PyBytes::new(py, &aggregated_ciphertexts).into()
+    aggregated_ciphertexts
 }
 
 #[pyfunction]
-fn randomize(py: Python, ciphertexts: PyObject) -> PyObject {
-    let ciphertexts_bytes: &PyBytes = ciphertexts
-        .cast_as(py)
-        .expect("The ciphertexts object should be of type `bytes`.");
-    let randomized_ciphertexts: Vec<u8> = ciphertexts_bytes
-        .as_bytes()
+fn randomize(_py: Python, ciphertexts: Vec<u8>) -> Vec<u8> {
+    let randomized_ciphertexts: Vec<u8> = ciphertexts
         .par_chunks(128)
         .flat_map(|chunk| WideCiphertext::from_bytes(chunk).randomize().to_bytes())
         .collect();
 
-    PyBytes::new(py, &randomized_ciphertexts).into()
+    randomized_ciphertexts
 }
 
 #[pyfunction]
-fn combine(py: Python, sender_ciphertexts: PyObject, receiver_ciphertexts: PyObject) -> &PyTuple {
-    let sender_ciphertexts_bytes: &PyBytes = sender_ciphertexts
-        .cast_as(py)
-        .expect("The sender_ciphertexts object should be of type `bytes`.");
-    let receiver_ciphertexts_bytes: &PyBytes = receiver_ciphertexts
-        .cast_as(py)
-        .expect("The receiver_ciphertexts object should be of type `bytes`.");
-
-    let wide_ciphertexts: Vec<WideCiphertext> = sender_ciphertexts_bytes
-        .as_bytes()
+fn combine(py: Python, sender_ciphertexts: Vec<u8>, receiver_ciphertexts: Vec<u8>) -> &PyTuple {
+    let wide_ciphertexts: Vec<WideCiphertext> = sender_ciphertexts
         .par_chunks(128)
-        .zip(receiver_ciphertexts_bytes.as_bytes().par_chunks(128))
+        .zip(receiver_ciphertexts.par_chunks(128))
         .map(|(x, y)| WideCiphertext::from_bytes(x).sum(WideCiphertext::from_bytes(y)))
         .collect();
 
@@ -136,31 +108,16 @@ fn combine(py: Python, sender_ciphertexts: PyObject, receiver_ciphertexts: PyObj
     let gamma_bytes: Vec<u8> = wide_ciphertexts.par_iter().flat_map(|c| c.c).collect();
     let delta_bytes: Vec<u8> = wide_ciphertexts.par_iter().flat_map(|c| c.d).collect();
 
-    let sender_points: PyObject = PyBytes::new(py, &alpha_bytes).into();
-    let receiver_points: PyObject = PyBytes::new(py, &beta_bytes).into();
-    let gamma_points: PyObject = PyBytes::new(py, &gamma_bytes).into();
-    let delta_points: PyObject = PyBytes::new(py, &delta_bytes).into();
-
-    PyTuple::new(
-        py,
-        [sender_points, receiver_points, gamma_points, delta_points],
-    )
+    PyTuple::new(py, [alpha_bytes, beta_bytes, gamma_bytes, delta_bytes])
 }
 
 #[pyfunction]
-fn decrypt(py: Python, points: PyObject, bank_secret_key: PyObject) -> PyObject {
-    let secret_key_bytes: &PyBytes = bank_secret_key
-        .cast_as(py)
-        .expect("The bank_secret_key object should be of type `bytes`.");
+fn decrypt(_py: Python, points: Vec<u8>, bank_secret_key: Vec<u8>) -> Vec<u8> {
     let mut secret_key_array = [0; 32];
-    secret_key_array.copy_from_slice(secret_key_bytes.as_bytes());
+    secret_key_array.copy_from_slice(&bank_secret_key);
     let secret_key = Scalar::from_bits(secret_key_array);
 
-    let points_bytes: &PyBytes = points
-        .cast_as(py)
-        .expect("The points object should be of type `bytes`.");
-    let decrypted_points: Vec<u8> = points_bytes
-        .as_bytes()
+    let decrypted_points: Vec<u8> = points
         .par_chunks(32)
         .flat_map(|chunk| {
             (secret_key * CompressedEdwardsY::from_slice(chunk).decompress().unwrap())
@@ -169,52 +126,32 @@ fn decrypt(py: Python, points: PyObject, bank_secret_key: PyObject) -> PyObject
         })
         .collect();
 
-    PyBytes::new(py, &decrypted_points).into()
+    decrypted_points
 }
 
 #[pyfunction]
 fn finish(
-    py: Python,
-    sender_points: PyObject,
-    receiver_points: PyObject,
-    gamma_points: PyObject,
-    delta_points: PyObject,
-    swift_secret_key: PyObject,
+    _py: Python,
+    sender_points: Vec<u8>,
+    receiver_points: Vec<u8>,
+    gamma_points: Vec<u8>,
+    delta_points: Vec<u8>,
+    swift_secret_key: Vec<u8>,
 ) -> Vec<bool> {
-    let secret_key_bytes: &PyBytes = swift_secret_key
-        .cast_as(py)
-        .expect("The swift_secret_key object should be of type `bytes`.");
     let mut secret_key_array = [0; 32];
-    secret_key_array.copy_from_slice(secret_key_bytes.as_bytes());
+    secret_key_array.copy_from_slice(&swift_secret_key);
     let secret_key = Scalar::from_bits(secret_key_array);
 
-    let sender_points_bytes: &PyBytes = sender_points
-        .cast_as(py)
-        .expect("The points object should be of type `bytes`.");
-    let receiver_points_bytes: &PyBytes = receiver_points
-        .cast_as(py)
-        .expect("The points object should be of type `bytes`.");
-    let gamma_points_bytes: &PyBytes = gamma_points
-        .cast_as(py)
-        .expect("The points object should be of type `bytes`.");
-    let delta_points_bytes: &PyBytes = delta_points
-        .cast_as(py)
-        .expect("The points object should be of type `bytes`.");
-
-    let sender_points_iter = sender_points_bytes
-        .as_bytes()
+    let sender_points_iter = sender_points
         .chunks(32)
         .map(|chunk| CompressedEdwardsY::from_slice(chunk).decompress().unwrap());
-    let receiver_points_iter = receiver_points_bytes
-        .as_bytes()
+    let receiver_points_iter = receiver_points
         .chunks(32)
         .map(|chunk| CompressedEdwardsY::from_slice(chunk).decompress().unwrap());
-    let gamma_points_iter = gamma_points_bytes
-        .as_bytes()
+    let gamma_points_iter = gamma_points
         .chunks(32)
         .map(|chunk| CompressedEdwardsY::from_slice(chunk).decompress().unwrap());
-    let delta_points_iter = delta_points_bytes
-        .as_bytes()
+    let delta_points_iter = delta_points
         .chunks(32)
         .map(|chunk| CompressedEdwardsY::from_slice(chunk).decompress().unwrap());
 
